@@ -19,6 +19,7 @@
 #include "IDBEvents.h"
 #include "IDBTransaction.h"
 #include "IndexedDatabaseManager.h"
+#include "ProfilerHelpers.h"
 #include "TransactionThreadPool.h"
 
 #include "ipc/IndexedDBChild.h"
@@ -33,7 +34,7 @@ IDBTransaction* gCurrentTransaction = nullptr;
 
 const uint32_t kProgressHandlerGranularity = 1000;
 
-class TransactionPoolEventTarget : public StackBasedEventTarget
+class MOZ_STACK_CLASS TransactionPoolEventTarget : public StackBasedEventTarget
 {
 public:
   NS_DECL_NSIEVENTTARGET
@@ -53,9 +54,9 @@ nsresult
 ConvertCloneReadInfosToArrayInternal(
                                 JSContext* aCx,
                                 nsTArray<StructuredCloneReadInfo>& aReadInfos,
-                                jsval* aResult)
+                                JS::MutableHandle<JS::Value> aResult)
 {
-  JSObject* array = JS_NewArrayObject(aCx, 0, nullptr);
+  JS::Rooted<JSObject*> array(aCx, JS_NewArrayObject(aCx, 0, nullptr));
   if (!array) {
     NS_WARNING("Failed to make array!");
     return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
@@ -71,20 +72,20 @@ ConvertCloneReadInfosToArrayInternal(
          index++) {
       StructuredCloneReadInfo& readInfo = aReadInfos[index];
 
-      jsval val;
+      JS::Rooted<JS::Value> val(aCx);
       if (!IDBObjectStore::DeserializeValue(aCx, readInfo, &val)) {
         NS_WARNING("Failed to decode!");
         return NS_ERROR_DOM_DATA_CLONE_ERR;
       }
 
-      if (!JS_SetElement(aCx, array, index, &val)) {
+      if (!JS_SetElement(aCx, array, index, val.address())) {
         NS_WARNING("Failed to set array element!");
         return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
       }
     }
   }
 
-  *aResult = OBJECT_TO_JSVAL(array);
+  aResult.setObject(*array);
   return NS_OK;
 }
 
@@ -102,7 +103,7 @@ HelperBase::~HelperBase()
       NS_WARN_IF_FALSE(mainThread, "Couldn't get the main thread!");
 
       if (mainThread) {
-        NS_ProxyRelease(mainThread, static_cast<nsIDOMEventTarget*>(request));
+        NS_ProxyRelease(mainThread, static_cast<EventTarget*>(request));
       }
     }
   }
@@ -111,18 +112,18 @@ HelperBase::~HelperBase()
 nsresult
 HelperBase::WrapNative(JSContext* aCx,
                        nsISupports* aNative,
-                       jsval* aResult)
+                       JS::MutableHandle<JS::Value> aResult)
 {
   NS_ASSERTION(aCx, "Null context!");
   NS_ASSERTION(aNative, "Null pointer!");
-  NS_ASSERTION(aResult, "Null pointer!");
+  NS_ASSERTION(aResult.address(), "Null pointer!");
   NS_ASSERTION(mRequest, "Null request!");
 
-  JSObject* global = mRequest->GetParentObject();
+  JS::Rooted<JSObject*> global(aCx, mRequest->GetParentObject());
   NS_ASSERTION(global, "This should never be null!");
 
   nsresult rv =
-    nsContentUtils::WrapNative(aCx, global, aNative, aResult);
+    nsContentUtils::WrapNative(aCx, global, aNative, aResult.address());
   NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
   return NS_OK;
@@ -191,6 +192,8 @@ NS_IMETHODIMP
 AsyncConnectionHelper::Run()
 {
   if (NS_IsMainThread()) {
+    PROFILER_MAIN_THREAD_LABEL("IndexedDB", "AsyncConnectionHelper::Run");
+
     if (mTransaction &&
         mTransaction->IsAborted()) {
       // Always fire a "error" event with ABORT_ERR if the transaction was
@@ -220,6 +223,11 @@ AsyncConnectionHelper::Run()
           if (NS_SUCCEEDED(mResultCode) && NS_FAILED(rv)) {
             mResultCode = rv;
           }
+
+          IDB_PROFILER_MARK("IndexedDB Request %llu: Running main thread "
+                            "response (rv = %lu)",
+                            "IDBRequest[%llu] MT Done",
+                            mRequest->GetSerialNumber(), mResultCode);
         }
 
         // Call OnError if the database had an error or if the OnSuccess
@@ -266,6 +274,13 @@ AsyncConnectionHelper::Run()
 
   NS_ASSERTION(IndexedDatabaseManager::IsMainProcess(), "Wrong process!");
 
+  PROFILER_LABEL("IndexedDB", "AsyncConnectionHelper::Run");
+
+  IDB_PROFILER_MARK_IF(mRequest,
+                       "IndexedDB Request %llu: Beginning database work",
+                       "IDBRequest[%llu] DT Start",
+                       mRequest->GetSerialNumber());
+
   nsresult rv = NS_OK;
   nsCOMPtr<mozIStorageConnection> connection;
 
@@ -341,6 +356,12 @@ AsyncConnectionHelper::Run()
 #endif
   }
 
+  IDB_PROFILER_MARK_IF(mRequest,
+                       "IndexedDB Request %llu: Finished database work "
+                       "(rv = %lu)",
+                       "IDBRequest[%llu] DT Done", mRequest->GetSerialNumber(),
+                       mResultCode);
+
   return NS_DispatchToMainThread(this, NS_DISPATCH_NORMAL);
 }
 
@@ -420,10 +441,10 @@ AsyncConnectionHelper::Init()
   return NS_OK;
 }
 
-already_AddRefed<nsDOMEvent>
-AsyncConnectionHelper::CreateSuccessEvent()
+already_AddRefed<nsIDOMEvent>
+AsyncConnectionHelper::CreateSuccessEvent(mozilla::dom::EventTarget* aOwner)
 {
-  return CreateGenericEvent(NS_LITERAL_STRING(SUCCESS_EVT_STR),
+  return CreateGenericEvent(mRequest, NS_LITERAL_STRING(SUCCESS_EVT_STR),
                             eDoesNotBubble, eNotCancelable);
 }
 
@@ -433,7 +454,9 @@ AsyncConnectionHelper::OnSuccess()
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(mRequest, "Null request!");
 
-  nsRefPtr<nsDOMEvent> event = CreateSuccessEvent();
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB", "AsyncConnectionHelper::OnSuccess");
+
+  nsRefPtr<nsIDOMEvent> event = CreateSuccessEvent(mRequest);
   if (!event) {
     NS_ERROR("Failed to create event!");
     return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
@@ -467,9 +490,11 @@ AsyncConnectionHelper::OnError()
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(mRequest, "Null request!");
 
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB", "AsyncConnectionHelper::OnError");
+
   // Make an error event and fire it at the target.
-  nsRefPtr<nsDOMEvent> event =
-    CreateGenericEvent(NS_LITERAL_STRING(ERROR_EVT_STR), eDoesBubble,
+  nsRefPtr<nsIDOMEvent> event =
+    CreateGenericEvent(mRequest, NS_LITERAL_STRING(ERROR_EVT_STR), eDoesBubble,
                        eCancelable);
   if (!event) {
     NS_ERROR("Failed to create event!");
@@ -508,11 +533,11 @@ AsyncConnectionHelper::OnError()
 
 nsresult
 AsyncConnectionHelper::GetSuccessResult(JSContext* aCx,
-                                        jsval* aVal)
+                                        JS::MutableHandle<JS::Value> aVal)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
-  *aVal = JSVAL_VOID;
+  aVal.setUndefined();
   return NS_OK;
 }
 
@@ -556,6 +581,11 @@ AsyncConnectionHelper::MaybeSendResponseToChildProcess(nsresult aResultCode)
     return Success_NotSent;
   }
 
+  IDB_PROFILER_MARK("IndexedDB Request %llu: Sending response to child "
+                    "process (rv = %lu)",
+                    "IDBRequest[%llu] MT Done",
+                    mRequest->GetSerialNumber(), aResultCode);
+
   return SendResponseToChildProcess(aResultCode);
 }
 
@@ -579,15 +609,13 @@ AsyncConnectionHelper::OnParentProcessRequestComplete(
 
 // static
 nsresult
-AsyncConnectionHelper::ConvertCloneReadInfosToArray(
+AsyncConnectionHelper::ConvertToArrayAndCleanup(
                                   JSContext* aCx,
                                   nsTArray<StructuredCloneReadInfo>& aReadInfos,
-                                  jsval* aResult)
+                                  JS::MutableHandle<JS::Value> aResult)
 {
   NS_ASSERTION(aCx, "Null context!");
-  NS_ASSERTION(aResult, "Null pointer!");
-
-  JSAutoRequest ar(aCx);
+  NS_ASSERTION(aResult.address(), "Null pointer!");
 
   nsresult rv = ConvertCloneReadInfosToArrayInternal(aCx, aReadInfos, aResult);
 
@@ -622,19 +650,22 @@ StackBasedEventTarget::QueryInterface(REFNSIID aIID,
 }
 
 NS_IMETHODIMP
-MainThreadEventTarget::Dispatch(nsIRunnable* aRunnable,
-                                uint32_t aFlags)
+ImmediateRunEventTarget::Dispatch(nsIRunnable* aRunnable,
+                                  uint32_t aFlags)
 {
   NS_ASSERTION(aRunnable, "Null pointer!");
 
-  nsCOMPtr<nsIRunnable> runnable = aRunnable;
-  return NS_DispatchToMainThread(aRunnable, aFlags);
+  nsCOMPtr<nsIRunnable> runnable(aRunnable);
+  DebugOnly<nsresult> rv =
+    runnable->Run();
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-MainThreadEventTarget::IsOnCurrentThread(bool* aIsOnCurrentThread)
+ImmediateRunEventTarget::IsOnCurrentThread(bool* aIsOnCurrentThread)
 {
-  *aIsOnCurrentThread = NS_IsMainThread();
+  *aIsOnCurrentThread = true;
   return NS_OK;
 }
 
