@@ -7,8 +7,6 @@
 #include "ImageLogging.h"
 #include "nsJPEGDecoder.h"
 
-#include "imgIContainerObserver.h"
-
 #include "nsIInputStream.h"
 
 #include "nspr.h"
@@ -35,11 +33,26 @@ namespace mozilla {
 namespace image {
 
 #if defined(PR_LOGGING)
-PRLogModuleInfo *gJPEGlog = PR_NewLogModule("JPEGDecoder");
-static PRLogModuleInfo *gJPEGDecoderAccountingLog = PR_NewLogModule("JPEGDecoderAccounting");
+static PRLogModuleInfo *
+GetJPEGLog()
+{
+  static PRLogModuleInfo *sJPEGLog;
+  if (!sJPEGLog)
+    sJPEGLog = PR_NewLogModule("JPEGDecoder");
+  return sJPEGLog;
+}
+
+static PRLogModuleInfo *
+GetJPEGDecoderAccountingLog()
+{
+  static PRLogModuleInfo *sJPEGDecoderAccountingLog;
+  if (!sJPEGDecoderAccountingLog)
+    sJPEGDecoderAccountingLog = PR_NewLogModule("JPEGDecoderAccounting");
+  return sJPEGDecoderAccountingLog;
+}
 #else
-#define gJPEGlog
-#define gJPEGDecoderAccountingLog
+#define GetJPEGLog()
+#define GetJPEGDecoderAccountingLog()
 #endif
 
 static qcms_profile*
@@ -67,8 +80,9 @@ METHODDEF(void) my_error_exit (j_common_ptr cinfo);
 #define MAX_JPEG_MARKER_LENGTH  (((uint32_t)1 << 16) - 1)
 
 
-nsJPEGDecoder::nsJPEGDecoder(RasterImage &aImage, imgIDecoderObserver* aObserver)
- : Decoder(aImage, aObserver)
+nsJPEGDecoder::nsJPEGDecoder(RasterImage& aImage, Decoder::DecodeStyle aDecodeStyle)
+ : Decoder(aImage)
+ , mDecodeStyle(aDecodeStyle)
 {
   mState = JPEG_HEADER;
   mReading = true;
@@ -90,7 +104,7 @@ nsJPEGDecoder::nsJPEGDecoder(RasterImage &aImage, imgIDecoderObserver* aObserver
 
   mCMSMode = 0;
 
-  PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+  PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
          ("nsJPEGDecoder::nsJPEGDecoder: Creating JPEG decoder %p",
           this));
 }
@@ -107,7 +121,7 @@ nsJPEGDecoder::~nsJPEGDecoder()
   if (mInProfile)
     qcms_profile_release(mInProfile);
 
-  PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+  PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
          ("nsJPEGDecoder::~nsJPEGDecoder: Destroying JPEG decoder %p",
           this));
 }
@@ -190,7 +204,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
       /* Error due to corrupt stream - return NS_OK and consume silently
          so that libpr0n doesn't throw away a partial image load */
       mState = JPEG_SINK_NON_JPEG_TRAILER;
-      PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
              ("} (setjmp returned NS_ERROR_FAILURE)"));
       return;
     } else {
@@ -199,23 +213,23 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
          mozilla is seconds away from falling flat on its face. */
       PostDecoderError(error_code);
       mState = JPEG_ERROR;
-      PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
              ("} (setjmp returned an error)"));
       return;
     }
   }
 
-  PR_LOG(gJPEGlog, PR_LOG_DEBUG,
+  PR_LOG(GetJPEGLog(), PR_LOG_DEBUG,
          ("[this=%p] nsJPEGDecoder::Write -- processing JPEG data\n", this));
 
   switch (mState) {
   case JPEG_HEADER:
   {
-    LOG_SCOPE(gJPEGlog, "nsJPEGDecoder::Write -- entering JPEG_HEADER case");
+    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- entering JPEG_HEADER case");
 
     /* Step 3: read file parameters with jpeg_read_header() */
     if (jpeg_read_header(&mInfo, TRUE) == JPEG_SUSPENDED) {
-      PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
              ("} (JPEG_SUSPENDED)"));
       return; /* I/O suspension */
     }
@@ -267,7 +281,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
       default:
         mState = JPEG_ERROR;
         PostDataError();
-        PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+        PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
                ("} (unknown colorpsace (1))"));
         return;
       }
@@ -284,7 +298,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
         default:
           mState = JPEG_ERROR;
           PostDataError();
-          PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+          PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
                  ("} (unknown colorpsace (2))"));
           return;
         }
@@ -341,7 +355,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
       default:
         mState = JPEG_ERROR;
         PostDataError();
-        PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+        PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
                ("} (unknown colorpsace (3))"));
         return;
         break;
@@ -350,37 +364,31 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
 
     /*
      * Don't allocate a giant and superfluous memory buffer
-     * when the image is a sequential JPEG.
+     * when not doing a progressive decode.
      */
-    mInfo.buffered_image = jpeg_has_multiple_scans(&mInfo);
+    mInfo.buffered_image = mDecodeStyle == PROGRESSIVE && jpeg_has_multiple_scans(&mInfo);
 
     /* Used to set up image size so arrays can be allocated */
     jpeg_calc_output_dimensions(&mInfo);
 
-    uint32_t imagelength;
-    if (NS_FAILED(mImage.EnsureFrame(0, 0, 0, mInfo.image_width, mInfo.image_height,
-                                     gfxASurface::ImageFormatRGB24,
-                                     &mImageData, &imagelength))) {
+    if (!mImageData) {
       mState = JPEG_ERROR;
       PostDecoderError(NS_ERROR_OUT_OF_MEMORY);
-      PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
              ("} (could not initialize image frame)"));
       return;
     }
 
-    PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+    PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
            ("        JPEGDecoderAccounting: nsJPEGDecoder::Write -- created image frame with %ux%u pixels",
             mInfo.image_width, mInfo.image_height));
 
-    // Tell the superclass we're starting a frame
-    PostFrameStart();
-
     mState = JPEG_START_DECOMPRESS;
   }
 
   case JPEG_START_DECOMPRESS:
   {
-    LOG_SCOPE(gJPEGlog, "nsJPEGDecoder::Write -- entering JPEG_START_DECOMPRESS case");
+    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- entering JPEG_START_DECOMPRESS case");
     /* Step 4: set parameters for decompression */
 
     /* FIXME -- Should reset dct_method and dither mode
@@ -394,7 +402,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
 
     /* Step 5: Start decompressor */
     if (jpeg_start_decompress(&mInfo) == FALSE) {
-      PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
              ("} (I/O suspension after jpeg_start_decompress())"));
       return; /* I/O suspension */
     }
@@ -408,13 +416,13 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
   {
     if (mState == JPEG_DECOMPRESS_SEQUENTIAL)
     {
-      LOG_SCOPE(gJPEGlog, "nsJPEGDecoder::Write -- JPEG_DECOMPRESS_SEQUENTIAL case");
+      LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- JPEG_DECOMPRESS_SEQUENTIAL case");
       
       bool suspend;
       OutputScanlines(&suspend);
       
       if (suspend) {
-        PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+        PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
                ("} (I/O suspension after OutputScanlines() - SEQUENTIAL)"));
         return; /* I/O suspension */
       }
@@ -429,7 +437,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
   {
     if (mState == JPEG_DECOMPRESS_PROGRESSIVE)
     {
-      LOG_SCOPE(gJPEGlog, "nsJPEGDecoder::Write -- JPEG_DECOMPRESS_PROGRESSIVE case");
+      LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- JPEG_DECOMPRESS_PROGRESSIVE case");
 
       int status;
       do {
@@ -450,7 +458,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
             scan--;
 
           if (!jpeg_start_output(&mInfo, scan)) {
-            PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+            PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
                    ("} (I/O suspension after jpeg_start_output() - PROGRESSIVE)"));
             return; /* I/O suspension */
           }
@@ -468,7 +476,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
                jpeg_start_output() multiple times for the same scan */
             mInfo.output_scanline = 0xffffff;
           }
-          PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+          PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
                  ("} (I/O suspension after OutputScanlines() - PROGRESSIVE)"));
           return; /* I/O suspension */
         }
@@ -476,7 +484,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
         if (mInfo.output_scanline == mInfo.output_height)
         {
           if (!jpeg_finish_output(&mInfo)) {
-            PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+            PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
                    ("} (I/O suspension after jpeg_finish_output() - PROGRESSIVE)"));
             return; /* I/O suspension */
           }
@@ -495,12 +503,12 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
 
   case JPEG_DONE:
   {
-    LOG_SCOPE(gJPEGlog, "nsJPEGDecoder::ProcessData -- entering JPEG_DONE case");
+    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::ProcessData -- entering JPEG_DONE case");
 
     /* Step 7: Finish decompression */
 
     if (jpeg_finish_decompress(&mInfo) == FALSE) {
-      PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
              ("} (I/O suspension after jpeg_finish_decompress() - DONE)"));
       return; /* I/O suspension */
     }
@@ -511,7 +519,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
     break;
   }
   case JPEG_SINK_NON_JPEG_TRAILER:
-    PR_LOG(gJPEGlog, PR_LOG_DEBUG,
+    PR_LOG(GetJPEGLog(), PR_LOG_DEBUG,
            ("[this=%p] nsJPEGDecoder::ProcessData -- entering JPEG_SINK_NON_JPEG_TRAILER case\n", this));
 
     break;
@@ -520,7 +528,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
     NS_ABORT_IF_FALSE(0, "Should always return immediately after error and not re-enter decoder");
   }
 
-  PR_LOG(gJPEGDecoderAccountingLog, PR_LOG_DEBUG,
+  PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
          ("} (end of function)"));
   return;
 }
@@ -528,7 +536,7 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
 void
 nsJPEGDecoder::NotifyDone()
 {
-  PostFrameStop();
+  PostFrameStop(FrameBlender::kFrameOpaque);
   PostDecodeDone();
 }
 
